// TinyPIM (c) 1999 Pablo Halpern. File AppointmentDisplayList.h

#ifndef AppointmentDisplayList_dot_h
#define AppointmentDisplayList_dot_h 1

#include "DisplayList.h"
#include "DateTime.h"

class DateBook;

// Specialized DisplayList for Appointment records.
class AppointmentDisplayList : public DisplayList
{
public:
  // Construct with a reference to the address book
  AppointmentDisplayList(DateBook& dateBook);

  DateTime currDate() const { return currDate_; }
  void currDate(DateTime dt);

  // List appointments for the day containing the specified DateTime
  void listDay(DateTime dt);

  // List appointments for the week containing the specified DateTime
  void listWeek(DateTime dt);

  // List all records that contain the specified string
  void listContainsString(const std::string&);

protected:
  // Display the specified appointment record in one-line format.
  // Implements pure virtual base-class function.
  virtual void displayRecord(int recordId);

  // Override base-class function to retrieve more records.
  virtual bool fetchMore(int startId, int numRecords,
                         std::vector<int>& result);

private:
  DateBook&	dateBook_;

  // Enumerate different listing modes
  enum listMode { dayMode, weekMode, stringMode };
  listMode      mode_;

  // DateTime used for dayMode and weekMode
  DateTime      currDate_;

  // String to use for stringMode.
  std::string   containsString_;
};

#endif // AppointmentDisplayList_dot_h
